@@ -126,7 +126,7 @@ sub setTsTranslations($$%)
 # Generate .ts files from .po files.
 sub generateTs()
 {
-	my @languages = ("de", "es", "fr", "ru", "it", "pl");
+	my @languages = ("de", "es", "fr", "ru", "it", "pl", "nl");
 	my $tmpdir = ".tsdir";
 	mkdir $tmpdir unless -d $tmpdir;
 	mkdir "po" unless -d "po";
@@ -339,7 +339,7 @@ if ($from_configure) {
 	}
 }
 
-my $config_h = "#define VERSION \"1.0\"\n";
+my $config_h = "#define VERSION \"1.1\"\n";
 my $config_pri;
 if ($prefix) {
 	$config_h .= "#define CFG_PREFIX \"$prefix\"\n";
