@@ -27,6 +27,7 @@
 #ifndef FRAME_H
 #define FRAME_H
 
+#include "formatreplacer.h"
 #include <qstring.h>
 #include <qvariant.h>
 #if QT_VERSION >= 0x040000
@@ -50,6 +51,7 @@ public:
 		FT_Track,
 		FT_Genre,
 		FT_LastV1Frame = FT_Genre,
+		FT_AlbumArtist,
 		FT_Arranger,
 		FT_Author,
 		FT_Bpm,
@@ -58,9 +60,12 @@ public:
 		FT_Copyright,
 		FT_Disc,
 		FT_EncodedBy,
+		FT_Grouping,
 		FT_Isrc,
 		FT_Language,
 		FT_Lyricist,
+		FT_Lyrics,
+		FT_Media,
 		FT_OriginalAlbum,
 		FT_OriginalArtist,
 		FT_OriginalDate,
@@ -68,6 +73,7 @@ public:
 		FT_Performer,
 		FT_Picture,
 		FT_Publisher,
+		FT_Remixer,
 		FT_Subtitle,
 		FT_Website,
 		FT_LastFrame = FT_Website,
@@ -281,6 +287,24 @@ public:
 	 */
 	static Type getTypeFromName(QString name);
 
+	/**
+	 * If a frame contains a string list as a value, it is stored in a single
+	 * string, separated by this special separator character.
+	 *
+	 * @return separator character.
+	 */
+	static char stringListSeparator() { return '|'; }
+
+	/**
+	 * Convert string (e.g. "track/total number of tracks") to number.
+	 *
+	 * @param str string to convert
+	 * @param ok  if not 0, true is returned here if conversion is ok
+	 *
+	 * @return number in string ignoring total after slash.
+	 */
+	static int numberWithoutTotal(const QString& str, bool* ok = 0);
+
 private:
 	friend class TaggedFile;
 	Type m_type;
@@ -310,12 +334,12 @@ public:
 	 */
 	void enableAll();
 
-  /**
-   * Check if all fields are true.
-   *
-   * @return true if all fields are true.
-   */
-  bool areAllEnabled() const;
+	/**
+	 * Check if all fields are true.
+	 *
+	 * @return true if all fields are true.
+	 */
+	bool areAllEnabled() const;
 
 	/**
 	 * Check if frame is enabled.
@@ -337,7 +361,12 @@ public:
 	void enable(Frame::Type type, const QString& name = QString(), bool en = true);
 
 private:
-	enum { FTM_AllFrames = (1 << (Frame::FT_LastFrame + 1)) - 1 };
+	class not_used { int num_frame_types_check[
+			Frame::FT_LastFrame == 31
+			? 1 : -1 ]; };
+	enum { FTM_AllFrames = 0xffffffff };
+	// if FTM_AllFrames is not 31, use
+	// enum { FTM_AllFrames = (1 << (Frame::FT_LastFrame + 1)) - 1 };
 	unsigned long m_enabledFrames;
 	std::set<QString> m_disabledOtherFrames;
 };
@@ -383,6 +412,21 @@ public:
 	 */
 	void removeDisabledFrames(const FrameFilter& flt);
 
+	/**
+	 * Copy frames which are empty or inactive from other frames.
+	 * This can be used to merge two frame collections.
+	 *
+	 * @param frames other frames
+	 */
+	void merge(const FrameCollection& frames);
+
+	/**
+	 * Check if the standard tags are empty or inactive.
+	 *
+	 * @return true if empty or inactive.
+	 */
+	bool isEmptyOrInactive() const;
+
 	/**
 	 * Find a frame by name.
 	 *
@@ -393,6 +437,192 @@ public:
 	 * @return iterator or end() if not found.
 	 */
 	iterator findByName(const QString& name) const;
+
+	/**
+	 * Get value by type.
+	 *
+	 * @param type type
+	 *
+	 * @return value, QString::null if not found.
+	 */
+	QString getValue(Frame::Type type) const;
+
+	/**
+	 * Set value by type.
+	 *
+	 * @param type type
+	 * @param value value, nothing is done if QString::null
+	 */
+	void setValue(Frame::Type type, const QString& value);
+
+	/**
+	 * Get integer value by type.
+	 *
+	 * @param type type
+	 *
+	 * @return value, 0 if empty, -1 if not found.
+	 */
+	int getIntValue(Frame::Type type) const;
+
+	/**
+	 * Set integer value by type.
+	 *
+	 * @param type type
+	 * @param value value, 0 to set empty, nothing is done if -1
+	 */
+	void setIntValue(Frame::Type type, int value);
+
+	/**
+	 * Get artist.
+	 *
+	 * @return artist, QString::null if not found.
+	 */
+	QString getArtist() const { return getValue(Frame::FT_Artist); }
+	
+	/**
+	 * Set artist.
+	 *
+	 * @param artist artist, nothing is done if QString::null
+	 */
+	void setArtist(const QString& artist) { setValue(Frame::FT_Artist, artist); }
+	
+	/**
+	 * Get album.
+	 *
+	 * @return album, QString::null if not found.
+	 */
+	QString getAlbum() const { return getValue(Frame::FT_Album); }
+	
+	/**
+	 * Set album.
+	 *
+	 * @param album album, nothing is done if QString::null
+	 */
+	void setAlbum(const QString& album) { setValue(Frame::FT_Album, album); }
+	
+	/**
+	 * Get title.
+	 *
+	 * @return title, QString::null if not found.
+	 */
+	QString getTitle() const { return getValue(Frame::FT_Title); }
+	
+	/**
+	 * Set title.
+	 *
+	 * @param title title, nothing is done if QString::null
+	 */
+	void setTitle(const QString& title) { setValue(Frame::FT_Title, title); }
+	
+	/**
+	 * Get comment.
+	 *
+	 * @return comment, QString::null if not found.
+	 */
+	QString getComment() const { return getValue(Frame::FT_Comment); }
+	
+	/**
+	 * Set comment.
+	 *
+	 * @param comment comment, nothing is done if QString::null
+	 */
+	void setComment(const QString& comment) { setValue(Frame::FT_Comment, comment); }
+	
+	/**
+	 * Get genre.
+	 *
+	 * @return genre, QString::null if not found.
+	 */
+	QString getGenre() const { return getValue(Frame::FT_Genre); }
+	
+	/**
+	 * Set genre.
+	 *
+	 * @param genre genre, nothing is done if QString::null
+	 */
+	void setGenre(const QString& genre) { setValue(Frame::FT_Genre, genre); }
+	
+	/**
+	 * Get track.
+	 *
+	 * @return track, -1 if not found.
+	 */
+	int getTrack() const { return getIntValue(Frame::FT_Track); }
+	
+	/**
+	 * Set track.
+	 *
+	 * @param track track, nothing is done if -1
+	 */
+	void setTrack(int track) { setIntValue(Frame::FT_Track, track); }
+	
+	/**
+	 * Get year.
+	 *
+	 * @return year, -1 if not found.
+	 */
+	int getYear() const { return getIntValue(Frame::FT_Date); }
+	
+	/**
+	 * Set year.
+	 *
+	 * @param year year, nothing is done if -1
+	 */
+	void setYear(int year) { setIntValue(Frame::FT_Date, year); }
+};
+
+
+/**
+ * Replaces frame format codes in a string.
+ */
+class FrameFormatReplacer : public FormatReplacer {
+public:
+	/**
+	 * Constructor.
+	 *
+	 * @param frames frame collection
+	 * @param str    string with format codes
+	 */
+	explicit FrameFormatReplacer(
+		const FrameCollection& frames, const QString& str = QString());
+
+	/**
+	 * Destructor.
+	 */
+	virtual ~FrameFormatReplacer();
+
+	/**
+	 * Get help text for supported format codes.
+	 *
+	 * @param onlyRows if true only the tr elements are returned,
+	 *                 not the surrounding table
+	 *
+	 * @return help text.
+	 */
+	static QString getToolTip(bool onlyRows = false);
+
+protected:
+	/**
+	 * Replace a format code (one character %c or multiple characters %{chars}).
+	 * Supported format fields:
+	 * %s title (song)
+	 * %l album
+	 * %a artist
+	 * %c comment
+	 * %y year
+	 * %t track, two digits, i.e. leading zero if < 10
+	 * %T track, without leading zeroes
+	 * %g genre
+	 *
+	 * @param code format code
+	 *
+	 * @return replacement string,
+	 *         QString::null if code not found.
+	 */
+	virtual QString getReplacement(const QString& code) const;
+
+private:
+	const FrameCollection& m_frames;
 };
 
 #endif // FRAME_H
