@@ -35,6 +35,7 @@
 #include <qpushbutton.h>
 #include <qprogressbar.h>
 #include <qimage.h>
+#include <qtextcodec.h>
 #include "qtcompatmac.h"
 #if QT_VERSION >= 0x040000
 #include <QCloseEvent>
@@ -88,7 +89,6 @@
 #include "exportdialog.h"
 #include "numbertracksdialog.h"
 #include "filterdialog.h"
-#include "standardtags.h"
 #include "rendirdialog.h"
 #include "filelistitem.h"
 #include "dirlist.h"
@@ -269,11 +269,11 @@ Kid3App::Kid3App() :
 #ifdef HAVE_QTDBUS
 	if (QDBusConnection::sessionBus().isConnected()) {
 		QString serviceName("net.sourceforge.kid3");
-    QDBusConnection::sessionBus().registerService(serviceName);
+		QDBusConnection::sessionBus().registerService(serviceName);
 #ifndef CONFIG_USE_KDE
 		serviceName += '-';
 		serviceName += QString::number(::getpid());
-    QDBusConnection::sessionBus().registerService(serviceName);
+		QDBusConnection::sessionBus().registerService(serviceName);
 #endif
 		new ScriptInterface(this);
 		if (!QDBusConnection::sessionBus().registerObject("/Kid3", this)) {
@@ -943,6 +943,28 @@ void Kid3App::saveOptions()
 #endif
 }
 
+/**
+ * Set the ID3v1 and ID3v2 text encodings from the configuration.
+ */
+static void setTextEncodings()
+{
+#if defined HAVE_ID3LIB || defined HAVE_TAGLIB
+	const QTextCodec* id3v1TextCodec =
+		Kid3App::s_miscCfg.m_textEncodingV1 != "ISO-8859-1" ?
+		QTextCodec::codecForName(Kid3App::s_miscCfg.m_textEncodingV1.QCM_latin1()) : 0;
+#endif
+#ifdef HAVE_ID3LIB
+	Mp3File::setDefaultTextEncoding(
+		static_cast<MiscConfig::TextEncoding>(Kid3App::s_miscCfg.m_textEncoding));
+	Mp3File::setTextCodecV1(id3v1TextCodec);
+#endif
+#ifdef HAVE_TAGLIB
+	TagLibFile::setDefaultTextEncoding(
+		static_cast<MiscConfig::TextEncoding>(Kid3App::s_miscCfg.m_textEncoding));
+	TagLibFile::setTextCodecV1(id3v1TextCodec);
+#endif
+}
+
 /**
  * Load application options.
  */
@@ -952,14 +974,7 @@ void Kid3App::readOptions()
 	if (s_miscCfg.m_nameFilter.isEmpty()) {
 		createFilterString(&s_miscCfg.m_nameFilter);
 	}
-#ifdef HAVE_ID3LIB
-	Mp3File::setDefaultTextEncoding(
-		static_cast<MiscConfig::TextEncoding>(s_miscCfg.m_textEncoding));
-#endif
-#ifdef HAVE_TAGLIB
-	TagLibFile::setDefaultTextEncoding(
-		static_cast<MiscConfig::TextEncoding>(s_miscCfg.m_textEncoding));
-#endif
+	setTextEncodings();
 	s_fnFormatCfg.readFromConfig(m_config);
 	s_id3FormatCfg.readFromConfig(m_config);
 	s_genCfg.readFromConfig(m_config);
@@ -1725,18 +1740,22 @@ void Kid3App::setupImportDialog()
 	while (mp3file != 0) {
 		mp3file->getFile()->readTags(false);
 		if (firstTrack) {
-			StandardTags st;
-			mp3file->getFile()->getStandardTagsV2(&st);
-			if (st.artist.isEmpty() && st.album.isEmpty()) {
-				mp3file->getFile()->getStandardTagsV1(&st);
+			FrameCollection frames;
+			mp3file->getFile()->getAllFramesV2(frames);
+			QString artist = frames.getArtist();
+			QString album = frames.getAlbum();
+			if (artist.isEmpty() && album.isEmpty()) {
+				mp3file->getFile()->getAllFramesV1(frames);
+				artist = frames.getArtist();
+				album = frames.getAlbum();
 			}
-			m_trackDataList.m_artist = st.artist;
-			m_trackDataList.m_album = st.album;
+			m_trackDataList.setArtist(artist);
+			m_trackDataList.setAlbum(album);
 			firstTrack = false;
 			tag1Supported = mp3file->getFile()->isTagV1Supported();
 		}
 		m_trackDataList.push_back(ImportTrackData(mp3file->getFile()->getAbsFilename(),
-                                            mp3file->getFile()->getDuration()));
+		                                          mp3file->getFile()->getDuration()));
 		mp3file = m_view->nextFileInDir();
 	}
 
@@ -1763,32 +1782,24 @@ void Kid3App::setupImportDialog()
 void Kid3App::getTagsFromImportDialog(bool destV1, bool destV2)
 {
 	slotStatusMsg(i18n("Import..."));
-	ImportTrackDataVector::const_iterator it = m_trackDataList.begin();
-	StandardTags st1, st2;
-	StandardTagsFilter flt(destV1 ?
-												 m_view->getFilterFromID3V1() :
-												 m_view->getFilterFromID3V2());
+	ImportTrackDataVector::iterator it = m_trackDataList.begin();
+	FrameFilter flt(destV1 ?
+	                m_view->frameTableV1()->getEnabledFrameFilter(true) :
+	                m_view->frameTableV2()->getEnabledFrameFilter(true));
 	bool no_selection = m_view->numFilesSelected() == 0;
 	FileListItem* mp3file = m_view->firstFileInDir();
 	while (mp3file != 0) {
-		mp3file->getFile()->readTags(false);
-		if (destV1) mp3file->getFile()->getStandardTagsV1(&st1);
-		if (destV2) mp3file->getFile()->getStandardTagsV2(&st2);
+		TaggedFile* taggedFile = mp3file->getFile();
+		taggedFile->readTags(false);
 		if (it != m_trackDataList.end()) {
-			if (destV1) (*it).copyActiveTags(st1);
-			if (destV2) (*it).copyActiveTags(st2);
+			it->removeDisabledFrames(flt);
+			formatFramesIfEnabled(*it);
+			if (destV1) taggedFile->setFramesV1(*it, false);
+			if (destV2) taggedFile->setFramesV2(*it, false);
 			++it;
 		} else {
 			break;
 		}
-		if (destV1) {
-			formatStandardTagsIfEnabled(&st1);
-			mp3file->getFile()->setStandardTagsV1(&st1, flt);
-		}
-		if (destV2) {
-			formatStandardTagsIfEnabled(&st2);
-			mp3file->getFile()->setStandardTagsV2(&st2, flt);
-		}
 		mp3file = m_view->nextFileInDir();
 	}
 	if (!no_selection) {
@@ -1918,6 +1929,35 @@ void Kid3App::slotImportMusicBrainz()
 #endif
 }
 
+#if defined HAVE_ID3LIB && defined HAVE_TAGLIB
+/**
+ * Read file with TagLib if it has an ID3v2.4 tag.
+ *
+ * @param item       file list item, can be updated
+ * @param taggedFile tagged file
+ *
+ * @return tagged file (can be new TagLibFile).
+ */
+static TaggedFile* readWithTagLibIfId3V24(FileListItem* item,
+																					TaggedFile* taggedFile)
+{
+	if (dynamic_cast<Mp3File*>(taggedFile) != 0 &&
+			!taggedFile->isChanged() &&
+			taggedFile->isTagInformationRead() && taggedFile->hasTagV2() &&
+			taggedFile->getTagFormatV2() == QString::null) {
+		TagLibFile* tagLibFile;
+		if ((tagLibFile = new TagLibFile(
+					 taggedFile->getDirInfo(),
+					 taggedFile->getFilename())) != 0) {
+			item->setFile(tagLibFile);
+			taggedFile = tagLibFile;
+			taggedFile->readTags(false);
+		}
+	}
+	return taggedFile;
+}
+#endif
+
 /**
  * Set data to be exported.
  *
@@ -1929,22 +1969,25 @@ void Kid3App::setExportData(int src)
 	if (m_exportDialog) {
 		ImportTrackDataVector trackDataVector;
 		FileListItem* mp3file = m_view->firstFileInDir();
-		bool firstTrack = true;
 		while (mp3file != 0) {
-			mp3file->getFile()->readTags(false);
-			ImportTrackData trackData(mp3file->getFile()->getAbsFilename(),
-																mp3file->getFile()->getDuration());
-			if (src == ExportDialog::SrcV1) {
-				mp3file->getFile()->getStandardTagsV1(&trackData);
-			} else {
-				mp3file->getFile()->getStandardTagsV2(&trackData);
+			TaggedFile* taggedFile = mp3file->getFile();
+			if (taggedFile) {
+				taggedFile->readTags(false);
+#if defined HAVE_ID3LIB && defined HAVE_TAGLIB
+				taggedFile = readWithTagLibIfId3V24(mp3file, taggedFile);
+#endif
+				ImportTrackData trackData(taggedFile->getAbsFilename(),
+																	taggedFile->getDuration());
+				trackData.setFileExtension(taggedFile->getFileExtension());
+				trackData.setTagFormatV1(taggedFile->getTagFormatV1());
+				trackData.setTagFormatV2(taggedFile->getTagFormatV2());
+				if (src == ExportDialog::SrcV1) {
+					taggedFile->getAllFramesV1(trackData);
+				} else {
+					taggedFile->getAllFramesV2(trackData);
+				}
+				trackDataVector.push_back(trackData);
 			}
-			if (firstTrack) {
-				trackDataVector.m_artist = trackData.artist;
-				trackDataVector.m_album = trackData.album;
-				firstTrack = false;
-			}
-			trackDataVector.push_back(trackData);
 			mp3file = m_view->nextFileInDir();
 		}
 		m_exportDialog->setExportData(trackDataVector);
@@ -2097,14 +2140,7 @@ void Kid3App::slotSettingsConfigure()
 			if (!s_miscCfg.m_markTruncations) {
 				m_view->frameTableV1()->markRows(0);
 			}
-#ifdef HAVE_ID3LIB
-			Mp3File::setDefaultTextEncoding(
-				static_cast<MiscConfig::TextEncoding>(s_miscCfg.m_textEncoding));
-#endif
-#ifdef HAVE_TAGLIB
-			TagLibFile::setDefaultTextEncoding(
-				static_cast<MiscConfig::TextEncoding>(s_miscCfg.m_textEncoding));
-#endif
+			setTextEncodings();
 #if QT_VERSION < 0x040000
 			m_view->frameTableV1()->triggerUpdateGenres();
 			m_view->frameTableV2()->triggerUpdateGenres();
@@ -2121,7 +2157,6 @@ void Kid3App::slotSettingsConfigure()
  */
 void Kid3App::slotApplyFilenameFormat()
 {
-	StandardTags st;
 	if (m_view->numFilesSelected() == 1) {
 		updateCurrentSelection();
 	}
@@ -2171,35 +2206,6 @@ void Kid3App::slotApplyId3Format()
 	updateGuiControls();
 }
 
-#if defined HAVE_ID3LIB && defined HAVE_TAGLIB
-/**
- * Read file with TagLib if it has an ID3v2.4 tag.
- *
- * @param item       file list item, can be updated
- * @param taggedFile tagged file
- *
- * @return tagged file (can be new TagLibFile).
- */
-static TaggedFile* readWithTagLibIfId3V24(FileListItem* item,
-																					TaggedFile* taggedFile)
-{
-	if (dynamic_cast<Mp3File*>(taggedFile) != 0 &&
-			!taggedFile->isChanged() &&
-			taggedFile->isTagInformationRead() && taggedFile->hasTagV2() &&
-			taggedFile->getTagFormatV2() == QString::null) {
-		TagLibFile* tagLibFile;
-		if ((tagLibFile = new TagLibFile(
-					 taggedFile->getDirInfo(),
-					 taggedFile->getFilename())) != 0) {
-			item->setFile(tagLibFile);
-			taggedFile = tagLibFile;
-			taggedFile->readTags(false);
-		}
-	}
-	return taggedFile;
-}
-#endif
-
 /**
  * Schedule actions to rename a directory.
  */
@@ -2722,8 +2728,18 @@ void Kid3App::updateGuiControls()
 	int num_files_selected = 0;
 	bool tagV1Supported = false;
 
+#if QT_VERSION >= 0x040000
+	QList<QTreeWidgetItem*> selItems = m_view->getFileList()->selectedItems();
+#endif
+
 	while (mp3file != 0) {
-		if (mp3file->isSelected()) {
+		if (
+#if QT_VERSION >= 0x040000
+			selItems.contains(mp3file)
+#else
+			mp3file->isSelected()
+#endif
+			) {
 			mp3file->setInSelection(true);
 			TaggedFile* taggedFile = mp3file->getFile();
 			if (taggedFile) {
@@ -2879,20 +2895,21 @@ void Kid3App::pasteTagsV2()
 void Kid3App::getTagsFromFilenameV1()
 {
 	updateCurrentSelection();
-	StandardTags st;
+	FrameCollection frames;
 	FileListItem* mp3file = m_view->firstFile();
 	bool multiselect = m_view->numFilesSelected() > 1;
-	StandardTagsFilter flt(m_view->getFilterFromID3V1());
+	FrameFilter flt(m_view->frameTableV1()->getEnabledFrameFilter(true));
 	while (mp3file != 0) {
 		if (mp3file->isInSelection()) {
 			if (!multiselect && m_view->isFilenameEditEnabled()) {
 				mp3file->getFile()->setFilename(
 					m_view->getFilename());
 			}
-			mp3file->getFile()->getStandardTagsV1(&st);
-			mp3file->getFile()->getTagsFromFilename(&st, m_view->getFilenameFormat());
-			formatStandardTagsIfEnabled(&st);
-			mp3file->getFile()->setStandardTagsV1(&st, flt);
+			mp3file->getFile()->getAllFramesV1(frames);
+			mp3file->getFile()->getTagsFromFilename(frames, m_view->getFilenameFormat());
+			frames.removeDisabledFrames(flt);
+			formatFramesIfEnabled(frames);
+			mp3file->getFile()->setFramesV1(frames);
 		}
 		mp3file = m_view->nextFile();
 	}
@@ -2908,20 +2925,21 @@ void Kid3App::getTagsFromFilenameV1()
 void Kid3App::getTagsFromFilenameV2()
 {
 	updateCurrentSelection();
-	StandardTags st;
+	FrameCollection frames;
 	FileListItem* mp3file = m_view->firstFile();
 	bool multiselect = m_view->numFilesSelected() > 1;
-	StandardTagsFilter flt(m_view->getFilterFromID3V2());
+	FrameFilter flt(m_view->frameTableV2()->getEnabledFrameFilter(true));
 	while (mp3file != 0) {
 		if (mp3file->isInSelection()) {
 			if (!multiselect && m_view->isFilenameEditEnabled()) {
 				mp3file->getFile()->setFilename(
 					m_view->getFilename());
 			}
-			mp3file->getFile()->getStandardTagsV2(&st);
-			mp3file->getFile()->getTagsFromFilename(&st, m_view->getFilenameFormat());
-			formatStandardTagsIfEnabled(&st);
-			mp3file->getFile()->setStandardTagsV2(&st, flt);
+			mp3file->getFile()->getAllFramesV2(frames);
+			mp3file->getFile()->getTagsFromFilename(frames, m_view->getFilenameFormat());
+			frames.removeDisabledFrames(flt);
+			formatFramesIfEnabled(frames);
+			mp3file->getFile()->setFramesV2(frames);
 		}
 		mp3file = m_view->nextFile();
 	}
@@ -2939,20 +2957,20 @@ void Kid3App::getTagsFromFilenameV2()
 void Kid3App::getFilenameFromTags(int tag_version)
 {
 	updateCurrentSelection();
-	StandardTags st;
+	FrameCollection frames;
 	FileListItem* mp3file = m_view->firstFile();
 	bool multiselect = m_view->numFilesSelected() > 1;
 	while (mp3file != 0) {
 		if (mp3file->isInSelection()) {
 			if (tag_version == 2) {
-				mp3file->getFile()->getStandardTagsV2(&st);
+				mp3file->getFile()->getAllFramesV2(frames);
 			}
 			else {
-				mp3file->getFile()->getStandardTagsV1(&st);
+				mp3file->getFile()->getAllFramesV1(frames);
 			}
-			if (!st.isEmptyOrInactive()) {
+			if (!frames.isEmptyOrInactive()) {
 				mp3file->getFile()->getFilenameFromTags(
-					&st, m_view->getFilenameFormat());
+					frames, m_view->getFilenameFormat());
 				formatFileNameIfEnabled(mp3file->getFile());
 				if (!multiselect) {
 					m_view->setFilename(
@@ -2972,14 +2990,15 @@ void Kid3App::getFilenameFromTags(int tag_version)
 void Kid3App::copyV1ToV2()
 {
 	updateCurrentSelection();
-	StandardTags st;
-	StandardTagsFilter flt(m_view->getFilterFromID3V2());
+	FrameCollection frames;
+	FrameFilter flt(m_view->frameTableV2()->getEnabledFrameFilter(true));
 	FileListItem* mp3file = m_view->firstFile();
 	while (mp3file != 0) {
 		if (mp3file->isInSelection()) {
-			mp3file->getFile()->getStandardTagsV1(&st);
-			formatStandardTagsIfEnabled(&st);
-			mp3file->getFile()->setStandardTagsV2(&st, flt);
+			mp3file->getFile()->getAllFramesV1(frames);
+			frames.removeDisabledFrames(flt);
+			formatFramesIfEnabled(frames);
+			mp3file->getFile()->setFramesV2(frames, false);
 		}
 		mp3file = m_view->nextFile();
 	}
@@ -2993,14 +3012,15 @@ void Kid3App::copyV1ToV2()
 void Kid3App::copyV2ToV1()
 {
 	updateCurrentSelection();
-	StandardTags st;
-	StandardTagsFilter flt(m_view->getFilterFromID3V1());
+	FrameCollection frames;
+	FrameFilter flt(m_view->frameTableV1()->getEnabledFrameFilter(true));
 	FileListItem* mp3file = m_view->firstFile();
 	while (mp3file != 0) {
 		if (mp3file->isInSelection()) {
-			mp3file->getFile()->getStandardTagsV2(&st);
-			formatStandardTagsIfEnabled(&st);
-			mp3file->getFile()->setStandardTagsV1(&st, flt);
+			mp3file->getFile()->getAllFramesV2(frames);
+			frames.removeDisabledFrames(flt);
+			formatFramesIfEnabled(frames);
+			mp3file->getFile()->setFramesV1(frames, false);
 		}
 		mp3file = m_view->nextFile();
 	}
@@ -3246,18 +3266,6 @@ void Kid3App::formatFileNameIfEnabled(TaggedFile* taggedFile) const
 	}
 }
 
-/**
- * Format tags if format while editing is switched on.
- *
- * @param st standard tags
- */
-void Kid3App::formatStandardTagsIfEnabled(StandardTags* st) const
-{
-	if (s_id3FormatCfg.m_formatWhileEditing) {
-		s_id3FormatCfg.formatStandardTags(*st);
-	}
-}
-
 /**
  * Format frames if format while editing is switched on.
  *
