@@ -29,6 +29,7 @@
 
 #include <qdir.h>
 #include <qstring.h>
+#include <qtextcodec.h>
 #include "qtcompatmac.h"
 #if QT_VERSION >= 0x040000
 #include <QByteArray>
@@ -39,10 +40,10 @@
 #include <id3.h>
 #endif
 
-#include "standardtags.h"
 #include "genres.h"
 #include "dirinfo.h"
 #include "kid3.h"
+#include <cstring>
 #include <sys/stat.h>
 #ifdef WIN32
 #include <sys/utime.h>
@@ -61,6 +62,9 @@
 #define UNICODE_SUPPORT_BUGGY ((((ID3LIB_MAJOR_VERSION) << 16) + ((ID3LIB_MINOR_VERSION) << 8) + (ID3LIB_PATCH_VERSION)) <= 0x030803)
 #endif
 
+/** Text codec for ID3v1 tags, 0 to use default (ISO 8859-1) */
+const QTextCodec* Mp3File::s_textCodecV1 = 0;
+
 /** Default text encoding */
 ID3_TextEnc Mp3File::s_defaultTextEncoding = ID3TE_ISO8859_1;
 
@@ -230,51 +234,93 @@ void Mp3File::deleteFramesV1(const FrameFilter& flt)
 	}
 }
 
+/**
+ * Fix up a unicode string from id3lib.
+ *
+ * @param str      unicode string
+ * @param numChars number of characters in str
+ *
+ * @return string as QString.
+ */
+static QString fixUpUnicode(const unicode_t* str, size_t numChars)
+{
+	QString text;
+	if (numChars > 0 && str && *str) {
+		QChar* qcarray = new QChar[numChars];
+		if (qcarray) {
+			// Unfortunately, Unicode support in id3lib is rather buggy
+			// in the current version: The codes are mirrored.
+			// In the hope that my patches will be included, I try here
+			// to work around these bugs.
+			size_t i;
+			size_t numZeroes = 0;
+			for (i = 0; i < numChars; i++) {
+				qcarray[i] =
+					UNICODE_SUPPORT_BUGGY ?
+					(ushort)(((str[i] & 0x00ff) << 8) |
+									 ((str[i] & 0xff00) >> 8)) :
+					(ushort)str[i];
+				if (qcarray[i].isNull()) { ++numZeroes; }
+			}
+			// remove a single trailing zero character
+			if (numZeroes == 1 && qcarray[numChars - 1].isNull()) {
+				--numChars;
+			}
+			text = QString(qcarray, numChars);
+			delete [] qcarray;
+		}
+	}
+	return text;
+}
+
 /**
  * Get string from text field.
  *
  * @param field field
+ * @param codec text codec to use, 0 for default
  *
  * @return string,
  *         "" if the field does not exist.
  */
-static QString getString(ID3_Field* field)
+static QString getString(ID3_Field* field, const QTextCodec* codec = 0)
 {
 	QString text("");
 	if (field != NULL) {
 		ID3_TextEnc enc = field->GetEncoding();
 		if (enc == ID3TE_UTF16 || enc == ID3TE_UTF16BE) {
-			const unicode_t* txt = field->GetRawUnicodeText();
-			uint unicode_size = field->Size() / sizeof(unicode_t);
-			if (unicode_size && txt) {
-				QChar* qcarray = new QChar[unicode_size];
-				if (qcarray) {
-					// Unfortunately, Unicode support in id3lib is rather buggy
-					// in the current version: The codes are mirrored.
-					// In the hope that my patches will be included, I try here
-					// to work around these bugs.
-					uint i;
-					uint numZeroes = 0;
-					for (i = 0; i < unicode_size; i++) {
-						qcarray[i] =
-							UNICODE_SUPPORT_BUGGY ?
-							(ushort)(((txt[i] & 0x00ff) << 8) |
-									 ((txt[i] & 0xff00) >> 8)) :
-							(ushort)txt[i];
-						if (qcarray[i].isNull()) { ++numZeroes; }
-					}
-					// remove a single trailing zero character
-					if (numZeroes == 1 && qcarray[unicode_size - 1].isNull()) {
-						--unicode_size;
-					}
-					text = QString(qcarray, unicode_size);
-					delete [] qcarray;
-				}
+			size_t numItems = field->GetNumTextItems();
+			if (numItems <= 1) {
+				text = fixUpUnicode(field->GetRawUnicodeText(),
+				                    field->Size() / sizeof(unicode_t));
+			} else {
+				// if there are multiple items, put them into one string
+				// separated by a special separator.
+				// ID3_Field::GetRawUnicodeTextItem() returns a pointer to a temporary
+				// object, so I do not use it.
+				text = fixUpUnicode(field->GetRawUnicodeText(),
+				                    field->Size() / sizeof(unicode_t));
+				text.replace('\0', Frame::stringListSeparator());
 			}
 		} else {
 			// (ID3TE_IS_SINGLE_BYTE_ENC(enc))
 			// (enc == ID3TE_ISO8859_1 || enc == ID3TE_UTF8)
-			text = QString(field->GetRawText());
+			size_t numItems = field->GetNumTextItems();
+			if (numItems <= 1) {
+				text = codec ?
+					codec->toUnicode(field->GetRawText(), field->Size()) :
+					QString(field->GetRawText());
+			} else {
+				// if there are multiple items, put them into one string
+				// separated by a special separator.
+				for (size_t itemNr = 0; itemNr < numItems; ++itemNr) {
+					if (itemNr == 0) {
+						text = field->GetRawTextItem(0);
+					} else {
+						text += Frame::stringListSeparator();
+						text += field->GetRawTextItem(itemNr);
+					}
+				}
+			}
 		}
 	}
 	return text;
@@ -285,11 +331,13 @@ static QString getString(ID3_Field* field)
  *
  * @param tag ID3 tag
  * @param id  frame ID
+ * @param codec text codec to use, 0 for default
  * @return string,
  *         "" if the field does not exist,
  *         QString::null if the tags do not exist.
  */
-static QString getTextField(const ID3_Tag* tag, ID3_FrameID id)
+static QString getTextField(const ID3_Tag* tag, ID3_FrameID id,
+														const QTextCodec* codec = 0)
 {
 	if (!tag) {
 		return QString::null;
@@ -298,7 +346,7 @@ static QString getTextField(const ID3_Tag* tag, ID3_FrameID id)
 	ID3_Field* fld;
 	ID3_Frame* frame = tag->Find(id);
 	if (frame && ((fld = frame->GetField(ID3FN_TEXT)) != NULL)) {
-		str = getString(fld);
+		str = getString(fld, codec);
 	}
 	return str;
 }
@@ -369,47 +417,115 @@ static int getGenreNum(const ID3_Tag* tag)
 	return n;
 }
 
+/**
+ * Allocate a fixed up a unicode string for id3lib.
+ *
+ * @param text string
+ *
+ * @return new allocated unicode string, has to be freed with delete [].
+ */
+static unicode_t* newFixedUpUnicode(const QString& text)
+{
+	// Unfortunately, Unicode support in id3lib is rather buggy in the
+	// current version: The codes are mirrored, a second different
+	// BOM may be added, if the LSB >= 0x80, the MSB is set to 0xff.
+	// If iconv is used (id3lib on Linux), the character do not come
+	// back mirrored, but with a second (different)! BOM 0xfeff and
+	// they are still written in the wrong order (big endian).
+	// In the hope that my patches will be included, I try here to
+	// work around these bugs, but there is no solution for the
+	// LSB >= 0x80 bug.
+	const QChar* qcarray = text.unicode();
+	uint unicode_size = text.length();
+	unicode_t* unicode = new unicode_t[unicode_size + 1];
+	if (unicode) {
+		uint i;
+		for (i = 0; i < unicode_size; i++) {
+			unicode[i] = (ushort)qcarray[i].unicode();
+			if (UNICODE_SUPPORT_BUGGY) {
+				unicode[i] = (ushort)(((unicode[i] & 0x00ff) << 8) |
+				                      ((unicode[i] & 0xff00) >> 8));
+			}
+		}
+		unicode[unicode_size] = 0;
+	}
+	return unicode;
+}
+
+/**
+ * Set string list in text field.
+ *
+ * @param field field
+ * @param lst   string list to set
+ */
+static void setStringList(ID3_Field* field, const QStringList& lst)
+{
+	ID3_TextEnc enc = field->GetEncoding();
+	bool first = true;
+	for (QStringList::const_iterator it = lst.begin(); it != lst.end(); ++it) {
+		if (first) {
+			if (enc == ID3TE_UTF16 || enc == ID3TE_UTF16BE) {
+				unicode_t* unicode = newFixedUpUnicode(*it);
+				if (unicode) {
+					field->Set(unicode);
+					delete [] unicode;
+				}
+			} else if (enc == ID3TE_UTF8) {
+				field->Set((*it).QCM_toUtf8().data());
+			} else {
+				// enc == ID3TE_ISO8859_1
+				field->Set((*it).QCM_latin1());
+			}
+			first = false;
+		} else {
+			// This will not work with buggy id3lib. A BOM 0xfffe is written before
+			// the first string, but not before the subsequent strings. Prepending a
+			// BOM or changing the byte order does not help when id3lib rewrites
+			// this field when another frame is changed. So you cannot use
+			// string lists with Unicode encoding.
+			if (enc == ID3TE_UTF16 || enc == ID3TE_UTF16BE) {
+				unicode_t* unicode = newFixedUpUnicode(*it);
+				if (unicode) {
+					field->Add(unicode);
+					delete [] unicode;
+				}
+			} else if (enc == ID3TE_UTF8) {
+				field->Add((*it).QCM_toUtf8().data());
+			} else {
+				// enc == ID3TE_ISO8859_1
+				field->Add((*it).QCM_latin1());
+			}
+		}
+	}
+}
+
 /**
  * Set string in text field.
  *
  * @param field        field
  * @param text         text to set
+ * @param codec        text codec to use, 0 for default
  */
-static void setString(ID3_Field* field, const QString& text)
+static void setString(ID3_Field* field, const QString& text,
+											const QTextCodec* codec = 0)
 {
-	ID3_TextEnc enc = field->GetEncoding();
-	// (ID3TE_IS_DOUBLE_BYTE_ENC(enc))
-	if (enc == ID3TE_UTF16 || enc == ID3TE_UTF16BE) {
-		// Unfortunately, Unicode support in id3lib is rather buggy in the
-		// current version: The codes are mirrored, a second different
-		// BOM may be added, if the LSB >= 0x80, the MSB is set to 0xff.
-		// If iconv is used (id3lib on Linux), the character do not come
-		// back mirrored, but with a second (different)! BOM 0xfeff and
-		// they are still written in the wrong order (big endian).
-		// In the hope that my patches will be included, I try here to
-		// work around these bugs, but there is no solution for the
-		// LSB >= 0x80 bug.
-		const QChar* qcarray = text.unicode();
-		uint unicode_size = text.length();
-		unicode_t* unicode = new unicode_t[unicode_size + 1];
-		if (unicode) {
-			uint i;
-			for (i = 0; i < unicode_size; i++) {
-				unicode[i] = (ushort)qcarray[i].unicode();
-				if (UNICODE_SUPPORT_BUGGY) {
-					unicode[i] = (ushort)(((unicode[i] & 0x00ff) << 8) |
-										  ((unicode[i] & 0xff00) >> 8));
-				}
+	if (text.QCM_indexOf(Frame::stringListSeparator()) == -1) {
+		ID3_TextEnc enc = field->GetEncoding();
+		// (ID3TE_IS_DOUBLE_BYTE_ENC(enc))
+		if (enc == ID3TE_UTF16 || enc == ID3TE_UTF16BE) {
+			unicode_t* unicode = newFixedUpUnicode(text);
+			if (unicode) {
+				field->Set(unicode);
+				delete [] unicode;
 			}
-			unicode[unicode_size] = 0;
-			field->Set(unicode);
-			delete [] unicode;
+		} else if (enc == ID3TE_UTF8) {
+			field->Set(text.QCM_toUtf8().data());
+		} else {
+			// enc == ID3TE_ISO8859_1
+			field->Set(codec ? codec->fromUnicode(text).data() : text.QCM_latin1());
 		}
-	} else if (enc == ID3TE_UTF8) {
-		field->Set(text.QCM_toUtf8().data());
 	} else {
-		// enc == ID3TE_ISO8859_1
-		field->Set(text.QCM_latin1());
+		setStringList(field, QCM_split(Frame::stringListSeparator(), text));
 	}
 }
 
@@ -422,12 +538,13 @@ static void setString(ID3_Field* field, const QString& text)
  * @param allowUnicode true to allow setting of Unicode encoding if necessary
  * @param replace      true to replace an existing field
  * @param removeEmpty  true to remove a field if text is empty
+ * @param codec        text codec to use, 0 for default
  *
  * @return true if the field was changed.
  */
 static bool setTextField(ID3_Tag* tag, ID3_FrameID id, const QString& text,
 												 bool allowUnicode = false, bool replace = true,
-												 bool removeEmpty = true)
+												 bool removeEmpty = true, const QTextCodec* codec = 0)
 {
 	bool changed = false;
 	if (tag && !text.isNull()) {
@@ -466,7 +583,7 @@ static bool setTextField(ID3_Tag* tag, ID3_FrameID id, const QString& text,
 						encfld->Set(enc);
 					}
 					fld->SetEncoding(enc);
-					setString(fld, text);
+					setString(fld, text, codec);
 					tag->AttachFrame(frame);
 				}
 			}
@@ -562,7 +679,7 @@ static bool setGenreNum(ID3_Tag* tag, int num)
  */
 QString Mp3File::getTitleV1()
 {
-	return getTextField(m_tagV1, ID3FID_TITLE);
+	return getTextField(m_tagV1, ID3FID_TITLE, s_textCodecV1);
 }
 
 /**
@@ -574,7 +691,7 @@ QString Mp3File::getTitleV1()
  */
 QString Mp3File::getArtistV1()
 {
-	return getTextField(m_tagV1, ID3FID_LEADARTIST);
+	return getTextField(m_tagV1, ID3FID_LEADARTIST, s_textCodecV1);
 }
 
 /**
@@ -586,7 +703,7 @@ QString Mp3File::getArtistV1()
  */
 QString Mp3File::getAlbumV1()
 {
-	return getTextField(m_tagV1, ID3FID_ALBUM);
+	return getTextField(m_tagV1, ID3FID_ALBUM, s_textCodecV1);
 }
 
 /**
@@ -598,7 +715,7 @@ QString Mp3File::getAlbumV1()
  */
 QString Mp3File::getCommentV1()
 {
-	return getTextField(m_tagV1, ID3FID_COMMENT);
+	return getTextField(m_tagV1, ID3FID_COMMENT, s_textCodecV1);
 }
 
 /**
@@ -740,11 +857,11 @@ QString Mp3File::getGenreV2()
  */
 void Mp3File::setTitleV1(const QString& str)
 {
-	if (getTextField(m_tagV1, ID3FID_TITLE) != str &&
-			setTextField(m_tagV1, ID3FID_TITLE, str)) {
+	if (getTextField(m_tagV1, ID3FID_TITLE, s_textCodecV1) != str &&
+			setTextField(m_tagV1, ID3FID_TITLE, str, false, true, true, s_textCodecV1)) {
 		markTag1Changed();
-		QString s = checkTruncation(str, StandardTags::TF_Title);
-		if (!s.isNull()) setTextField(m_tagV1, ID3FID_TITLE, s);
+		QString s = checkTruncation(str, 1 << Frame::FT_Title);
+		if (!s.isNull()) setTextField(m_tagV1, ID3FID_TITLE, s, false, true, true, s_textCodecV1);
 	}
 }
 
@@ -755,11 +872,11 @@ void Mp3File::setTitleV1(const QString& str)
  */
 void Mp3File::setArtistV1(const QString& str)
 {
-	if (getTextField(m_tagV1, ID3FID_LEADARTIST) != str &&
-			setTextField(m_tagV1, ID3FID_LEADARTIST, str)) {
+	if (getTextField(m_tagV1, ID3FID_LEADARTIST, s_textCodecV1) != str &&
+			setTextField(m_tagV1, ID3FID_LEADARTIST, str, false, true, true, s_textCodecV1)) {
 		markTag1Changed();
-		QString s = checkTruncation(str, StandardTags::TF_Artist);
-		if (!s.isNull()) setTextField(m_tagV1, ID3FID_LEADARTIST, s);
+		QString s = checkTruncation(str, 1 << Frame::FT_Artist);
+		if (!s.isNull()) setTextField(m_tagV1, ID3FID_LEADARTIST, s, false, true, true, s_textCodecV1);
 	}
 }
 
@@ -770,11 +887,11 @@ void Mp3File::setArtistV1(const QString& str)
  */
 void Mp3File::setAlbumV1(const QString& str)
 {
-	if (getTextField(m_tagV1, ID3FID_ALBUM) != str &&
-			setTextField(m_tagV1, ID3FID_ALBUM, str)) {
+	if (getTextField(m_tagV1, ID3FID_ALBUM, s_textCodecV1) != str &&
+			setTextField(m_tagV1, ID3FID_ALBUM, str, false, true, true, s_textCodecV1)) {
 		markTag1Changed();
-		QString s = checkTruncation(str, StandardTags::TF_Album);
-		if (!s.isNull()) setTextField(m_tagV1, ID3FID_ALBUM, s);
+		QString s = checkTruncation(str, 1 << Frame::FT_Album);
+		if (!s.isNull()) setTextField(m_tagV1, ID3FID_ALBUM, s, false, true, true, s_textCodecV1);
 	}
 }
 
@@ -785,11 +902,11 @@ void Mp3File::setAlbumV1(const QString& str)
  */
 void Mp3File::setCommentV1(const QString& str)
 {
-	if (getTextField(m_tagV1, ID3FID_COMMENT) != str &&
-			setTextField(m_tagV1, ID3FID_COMMENT, str)) {
+	if (getTextField(m_tagV1, ID3FID_COMMENT, s_textCodecV1) != str &&
+			setTextField(m_tagV1, ID3FID_COMMENT, str, false, true, true, s_textCodecV1)) {
 		markTag1Changed();
-		QString s = checkTruncation(str, StandardTags::TF_Comment, 28);
-		if (!s.isNull()) setTextField(m_tagV1, ID3FID_COMMENT, s);
+		QString s = checkTruncation(str, 1 << Frame::FT_Comment, 28);
+		if (!s.isNull()) setTextField(m_tagV1, ID3FID_COMMENT, s, false, true, true, s_textCodecV1);
 	}
 }
 
@@ -814,7 +931,7 @@ void Mp3File::setTrackNumV1(int num)
 {
 	if (setTrackNum(m_tagV1, num)) {
 		markTag1Changed();
-		int n = checkTruncation(num, StandardTags::TF_Track);
+		int n = checkTruncation(num, 1 << Frame::FT_Track);
 		if (n != -1) setTrackNum(m_tagV1, n);
 	}
 }
@@ -833,7 +950,7 @@ void Mp3File::setGenreV1(const QString& str)
 		}
 		// if the string cannot be converted to a number, set the truncation flag
 		checkTruncation(num == 0xff && !str.isEmpty() ? 1 : 0,
-										StandardTags::TF_Genre, 0);
+										1 << Frame::FT_Genre, 0);
 	}
 }
 
@@ -1143,7 +1260,7 @@ static const struct TypeStrOfId {
 	{ Frame::FT_Other,          I18N_NOOP("ETCO - Event timing codes") },                              /* ETCO */
 	{ Frame::FT_Other,          I18N_NOOP("GEOB - General encapsulated object") },                     /* GEOB */
 	{ Frame::FT_Other,          I18N_NOOP("GRID - Group identification registration") },               /* GRID */
-	{ Frame::FT_Other,          I18N_NOOP("IPLS - Involved people list") },                            /* IPLS */
+	{ Frame::FT_Arranger,       I18N_NOOP("IPLS - Involved people list") },                            /* IPLS */
 	{ Frame::FT_Other,          I18N_NOOP("LINK - Linked information") },                              /* LINK */
 	{ Frame::FT_Other,          I18N_NOOP("MCDI - Music CD identifier") },                             /* MCDI */
 	{ Frame::FT_Other,          I18N_NOOP("MLLT - MPEG location lookup table") },                      /* MLLT */
@@ -1177,14 +1294,14 @@ static const struct TypeStrOfId {
 	{ Frame::FT_Lyricist,       I18N_NOOP("TEXT - Lyricist/Text writer") },                            /* TEXT */
 	{ Frame::FT_Other,          I18N_NOOP("TFLT - File type") },                                       /* TFLT */
 	{ Frame::FT_Other,          I18N_NOOP("TIME - Time") },                                            /* TIME */
-	{ Frame::FT_Other,          I18N_NOOP("TIT1 - Content group description") },                       /* TIT1 */
+	{ Frame::FT_Grouping,       I18N_NOOP("TIT1 - Content group description") },                       /* TIT1 */
 	{ Frame::FT_Title,          I18N_NOOP("TIT2 - Title/songname/content description") },              /* TIT2 */
 	{ Frame::FT_Subtitle,       I18N_NOOP("TIT3 - Subtitle/Description refinement") },                 /* TIT3 */
 	{ Frame::FT_Other,          I18N_NOOP("TKEY - Initial key") },                                     /* TKEY */
 	{ Frame::FT_Language,       I18N_NOOP("TLAN - Language(s)") },                                     /* TLAN */
 	{ Frame::FT_Other,          I18N_NOOP("TLEN - Length") },                                          /* TLEN */
 	{ Frame::FT_Other,          0 },                                                                   /* TMCL */
-	{ Frame::FT_Other,          I18N_NOOP("TMED - Media type") },                                      /* TMED */
+	{ Frame::FT_Media,          I18N_NOOP("TMED - Media type") },                                      /* TMED */
 	{ Frame::FT_Other,          0 },                                                                   /* TMOO */
 	{ Frame::FT_OriginalAlbum,  I18N_NOOP("TOAL - Original album/movie/show title") },                 /* TOAL */
 	{ Frame::FT_Other,          I18N_NOOP("TOFN - Original filename") },                               /* TOFN */
@@ -1193,9 +1310,9 @@ static const struct TypeStrOfId {
 	{ Frame::FT_OriginalDate,   I18N_NOOP("TORY - Original release year") },                           /* TORY */
 	{ Frame::FT_Other,          I18N_NOOP("TOWN - File owner/licensee") },                             /* TOWN */
 	{ Frame::FT_Artist,         I18N_NOOP("TPE1 - Lead performer(s)/Soloist(s)") },                    /* TPE1 */
-	{ Frame::FT_Performer,      I18N_NOOP("TPE2 - Band/orchestra/accompaniment") },                    /* TPE2 */
+	{ Frame::FT_AlbumArtist,    I18N_NOOP("TPE2 - Band/orchestra/accompaniment") },                    /* TPE2 */
 	{ Frame::FT_Conductor,      I18N_NOOP("TPE3 - Conductor/performer refinement") },                  /* TPE3 */
-	{ Frame::FT_Arranger,       I18N_NOOP("TPE4 - Interpreted, remixed, or otherwise modified by") },  /* TPE4 */
+	{ Frame::FT_Remixer,        I18N_NOOP("TPE4 - Interpreted, remixed, or otherwise modified by") },  /* TPE4 */
 	{ Frame::FT_Disc,           I18N_NOOP("TPOS - Part of a set") },                                   /* TPOS */
 	{ Frame::FT_Other,          0 },                                                                   /* TPRO */
 	{ Frame::FT_Publisher,      I18N_NOOP("TPUB - Publisher") },                                       /* TPUB */
@@ -1214,7 +1331,7 @@ static const struct TypeStrOfId {
 	{ Frame::FT_Date,           I18N_NOOP("TYER - Year") },                                            /* TYER */
 	{ Frame::FT_Other,          I18N_NOOP("UFID - Unique file identifier") },                          /* UFID */
 	{ Frame::FT_Other,          I18N_NOOP("USER - Terms of use") },                                    /* USER */
-	{ Frame::FT_Other,          I18N_NOOP("USLT - Unsynchronized lyric/text transcription") },         /* USLT */
+	{ Frame::FT_Lyrics,         I18N_NOOP("USLT - Unsynchronized lyric/text transcription") },         /* USLT */
 	{ Frame::FT_Other,          I18N_NOOP("WCOM - Commercial information") },                          /* WCOM */
 	{ Frame::FT_Other,          I18N_NOOP("WCOP - Copyright/Legal information") },                     /* WCOP */
 	{ Frame::FT_Other,          I18N_NOOP("WOAF - Official audio file webpage") },                     /* WOAF */
@@ -1255,6 +1372,11 @@ static void getTypeStringForId3libFrameId(ID3_FrameID id, Frame::Type& type,
  */
 static ID3_FrameID getId3libFrameIdForType(Frame::Type type)
 {
+	// IPLS is mapped to FT_Arranger and FT_Performer
+	if (type == Frame::FT_Performer) {
+		return ID3FID_INVOLVEDPEOPLE;
+	}
+
 	static int typeIdMap[Frame::FT_LastFrame + 1] = { -1, };
 	if (typeIdMap[0] == -1) {
 		// first time initialization
@@ -1402,10 +1524,11 @@ void Mp3File::setId3v2Frame(ID3_Frame* id3Frame, const Frame& frame) const
 			case QVariant::UInt:
 			{
 				int intVal = fld.m_value.toInt();
-				id3Field->Set(intVal);
 				if (fld.m_id == ID3FN_TEXTENC) {
+					if (intVal == ID3TE_UTF8) intVal = ID3TE_UTF16;
 					enc = static_cast<ID3_TextEnc>(intVal);
 				}
+				id3Field->Set(intVal);
 				break;
 			}
 
@@ -1714,13 +1837,23 @@ QStringList Mp3File::getFrameIds() const
 		if (typeStrOfId[i].type == Frame::FT_Other) {
 			const char* s = typeStrOfId[i].str;
 			if (s) {
-				lst.append(s);
+				lst.append(QCM_translate(s));
 			}
 		}
 	}
 	return lst;
 }
 
+/**
+ * Set the text codec to be used for tag 1.
+ *
+ * @param codec text codec, 0 to use default (ISO 8859-1)
+ */
+void Mp3File::setTextCodecV1(const QTextCodec* codec)
+{
+	s_textCodecV1 = codec;
+}
+
 /**
  * Set the default text encoding.
  *
