@@ -27,7 +27,6 @@
 #include "oggfile.hpp"
 #if defined HAVE_VORBIS || defined HAVE_FLAC
 
-#include "standardtags.h"
 #include "dirinfo.h"
 #include <qfile.h>
 #include <qdir.h>
@@ -245,6 +244,7 @@ static const char* getVorbisNameFromType(Frame::Type type)
 		"TRACKNUMBER",     // FT_Track,
 		"GENRE",           // FT_Genre,
 		                   // FT_LastV1Frame = FT_Track,
+		"ALBUMARTIST",     // FT_AlbumArtist,
 		"ARRANGER",        // FT_Arranger,
 		"AUTHOR",          // FT_Author,
 		"BPM",             // FT_Bpm,
@@ -253,9 +253,12 @@ static const char* getVorbisNameFromType(Frame::Type type)
 		"COPYRIGHT",       // FT_Copyright,
 		"DISCNUMBER",      // FT_Disc,
 		"ENCODED-BY",      // FT_EncodedBy,
+		"GROUPING",        // FT_Grouping,
 		"ISRC",            // FT_Isrc,
 		"LANGUAGE",        // FT_Language,
 		"LYRICIST",        // FT_Lyricist,
+		"LYRICS",          // FT_Lyrics,
+		"SOURCEMEDIA",     // FT_Media,
 		"ORIGINALALBUM",   // FT_OriginalAlbum,
 		"ORIGINALARTIST",  // FT_OriginalArtist,
 		"ORIGINALDATE",    // FT_OriginalDate,
@@ -263,6 +266,7 @@ static const char* getVorbisNameFromType(Frame::Type type)
 		"PERFORMER",       // FT_Performer,
 		"UNKNOWN",         // FT_Picture,
 		"PUBLISHER",       // FT_Publisher,
+		"REMIXER",         // FT_Remixer,
 		"SUBTITLE",        // FT_Subtitle,
 		"WEBSITE",         // FT_Website,
 		                   // FT_LastFrame = FT_Website
@@ -299,6 +303,23 @@ static Frame::Type getTypeFromVorbisName(QString name)
 	return Frame::FT_Other;
 }
 
+/**
+ * Get internal name of a Vorbis frame.
+ *
+ * @param frame frame
+ *
+ * @return Vorbis key.
+ */
+static QString getVorbisName(const Frame& frame)
+{
+	Frame::Type type = frame.getType();
+	if (type <= Frame::FT_LastFrame) {
+		return getVorbisNameFromType(type);
+	} else {
+		return frame.getName().remove(' ').QCM_toUpper();
+	}
+}
+
 /**
  * Remove ID3v2 frames.
  *
@@ -631,6 +652,9 @@ QString OggFile::getTagFormatV2() const
  */
 bool OggFile::setFrameV2(const Frame& frame)
 {
+	if (frame.getType() == Frame::FT_Picture) {
+		return false;
+	}
 	if (frame.getType() == Frame::FT_Track) {
 		int numTracks = getTotalNumberOfTracksIfEnabled();
 		if (numTracks > 0) {
@@ -677,8 +701,11 @@ bool OggFile::setFrameV2(const Frame& frame)
  */
 bool OggFile::addFrameV2(Frame& frame)
 {
+	if (frame.getType() == Frame::FT_Picture) {
+		return false;
+	}
 	// Add a new frame.
-	QString name(frame.getName().remove(' ').QCM_toUpper());
+	QString name(getVorbisName(frame));
 	m_comments.push_back(OggFile::CommentField(name, frame.getValue()));
 	frame.setInternalName(name);
 	frame.setIndex(m_comments.size() - 1);
@@ -740,7 +767,6 @@ void OggFile::getAllFramesV2(FrameCollection& frames)
 QStringList OggFile::getFrameIds() const
 {
 	static const char* const fieldNames[] = {
-		"ALBUMARTIST",
 		"CATALOGNUMBER",
 		"CONTACT",
 		"DESCRIPTION",
@@ -760,11 +786,9 @@ QStringList OggFile::getFrameIds() const
 		"PRODUCTNUMBER",
 		"RECORDINGDATE",
 		"RELEASE DATE",
-		"REMIXER",
 		"SOURCE ARTIST",
 		"SOURCE MEDIUM",
 		"SOURCE WORK",
-		"SOURCEMEDIA",
 		"SPARS",
 		"TRACKTOTAL",
 		"VERSION",
@@ -883,7 +907,8 @@ bool OggFile::CommentList::setValue(const QString& name, const QString& value)
 TaggedFile* OggFile::Resolver::createFile(const DirInfo* di,
 																					const QString& fn) const
 {
-	if (fn.right(4).QCM_toLower() == ".ogg")
+	QString ext = fn.right(4).QCM_toLower();
+	if (ext == ".oga" || ext == ".ogg")
 		return new OggFile(di, fn);
 	else
 		return 0;
@@ -896,7 +921,7 @@ TaggedFile* OggFile::Resolver::createFile(const DirInfo* di,
  */
 QStringList OggFile::Resolver::getSupportedFileExtensions() const
 {
-	return QStringList() << ".ogg";
+	return QStringList() << ".oga" << ".ogg";
 }
 
 #endif // HAVE_VORBIS || define HAVE_FLAC
