@@ -78,6 +78,9 @@
 #define KCM_readListEntry(key) readEntry(key, QStringList())
 #define KCM_readIntListEntry(key) readEntry(key, QList<int>())
 
+#define KCM_i18n1(s, a1) i18n(s, a1)
+#define KCM_i18n2(s, a1, a2) i18n(s, a1, a2)
+
 #else
 
 #define KCM_ICON_document_open "fileopen"
@@ -112,11 +115,16 @@
 #define KCM_readListEntry(key) readListEntry(key)
 #define KCM_readIntListEntry(key) readIntListEntry(key)
 
+#define KCM_i18n1(s, a1) i18n(s).arg(a1)
+#define KCM_i18n2(s, a1, a2) i18n(s).arg(a1).arg(a2)
+
 #endif
 #define QCM_translate(s) i18n(s)
 #else
 #define i18n(s) tr(s)
 #define I18N_NOOP(s) QT_TRANSLATE_NOOP("@default", s)
+#define KCM_i18n1(s, a1) tr(s).arg(a1)
+#define KCM_i18n2(s, a1, a2) tr(s).arg(a1).arg(a2)
 #if QT_VERSION >= 0x040000
 #include <QCoreApplication>
 #define QCM_translate(s) QCoreApplication::translate("@default", s)
