@@ -40,7 +40,7 @@
 #endif
 
 #include "taggedfile.h"
-#include "standardtags.h"
+#include "frame.h"
 #include "kid3.h"
 #include "miscconfig.h"
 #include "rendirdialog.h"
@@ -292,7 +292,7 @@ bool RenDirDialog::createDirectory(const QString& dir,
 		return true;
 	} else {
 		if (errorMsg) {
-			errorMsg->append(i18n("Create directory %1 failed\n").arg(dir));
+			errorMsg->append(KCM_i18n1("Create directory %1 failed\n", dir));
 		}
 		return false;
 	}
@@ -313,13 +313,13 @@ bool RenDirDialog::renameDirectory(
 {
 	if (QFileInfo(newdir).exists()) {
 		if (errorMsg) {
-			errorMsg->append(i18n("File %1 already exists\n").arg(newdir));
+			errorMsg->append(KCM_i18n1("File %1 already exists\n", newdir));
 		}
 		return false;
 	}
 	if (!QFileInfo(olddir).isDir()) {
 		if (errorMsg) {
-			errorMsg->append(i18n("%1 is not a directory\n").arg(olddir));
+			errorMsg->append(KCM_i18n1("%1 is not a directory\n", olddir));
 		}
 		return false;
 	}
@@ -327,7 +327,7 @@ bool RenDirDialog::renameDirectory(
 		return true;
 	} else {
 		if (errorMsg) {
-			errorMsg->append(i18n("Rename %1 to %2 failed\n").arg(olddir).arg(newdir));
+			errorMsg->append(KCM_i18n2("Rename %1 to %2 failed\n", olddir, newdir));
 		}
 		return false;
 	}
@@ -351,13 +351,13 @@ bool RenDirDialog::renameFile(const QString& oldfn, const QString& newfn,
 	}
 	if (QFileInfo(newfn).exists()) {
 		if (errorMsg) {
-			errorMsg->append(i18n("%1 already exists\n").arg(newfn));
+			errorMsg->append(KCM_i18n1("%1 already exists\n", newfn));
 		}
 		return false;
 	}
 	if (!QFileInfo(oldfn).isFile()) {
 		if (errorMsg) {
-			errorMsg->append(i18n("%1 is not a file\n").arg(oldfn));
+			errorMsg->append(KCM_i18n1("%1 is not a file\n", oldfn));
 		}
 		return false;
 	}
@@ -365,7 +365,7 @@ bool RenDirDialog::renameFile(const QString& oldfn, const QString& newfn,
 		return true;
 	} else {
 		if (errorMsg) {
-			errorMsg->append(i18n("Rename %1 to %2 failed\n").arg(oldfn).arg(newfn));
+			errorMsg->append(KCM_i18n2("Rename %1 to %2 failed\n", oldfn, newfn));
 		}
 		return false;
 	}
@@ -382,23 +382,23 @@ bool RenDirDialog::renameFile(const QString& oldfn, const QString& newfn,
  */
 QString RenDirDialog::generateNewDirname(TaggedFile* taggedFile, QString* olddir)
 {
-	StandardTags st;
+	FrameCollection frames;
 	taggedFile->readTags(false);
 	switch (m_tagversionComboBox->QCM_currentIndex()) {
 		case TagV1:
-			taggedFile->getStandardTagsV1(&st);
+			taggedFile->getAllFramesV1(frames);
 			break;
 		case TagV2:
-			taggedFile->getStandardTagsV2(&st);
+			taggedFile->getAllFramesV2(frames);
 			break;
 		case TagV2V1:
 		default:
 		{
 			// use merged tags 1 and 2
-			StandardTags st1;
-			taggedFile->getStandardTagsV1(&st1);
-			taggedFile->getStandardTagsV2(&st);
-			st.merge(st1);
+			FrameCollection frames1;
+			taggedFile->getAllFramesV1(frames1);
+			taggedFile->getAllFramesV2(frames);
+			frames.merge(frames1);
 		}
 	}
 	QString newdir(taggedFile->getDirname());
@@ -412,13 +412,14 @@ QString RenDirDialog::generateNewDirname(TaggedFile* taggedFile, QString* olddir
 	if (olddir) {
 		*olddir = newdir;
 	}
-	if (!st.isEmptyOrInactive()) {
+	if (!frames.isEmptyOrInactive()) {
 		if (m_actionComboBox->QCM_currentIndex() == ActionRename) {
 			newdir = parentDirectory(newdir);
 		} else if (!newdir.isEmpty()) {
 			newdir.append('/');
 		}
-		newdir.append(taggedFile->formatWithTags(&st, m_formatComboBox->currentText(), true));
+		newdir.append(taggedFile->formatWithTags(
+										frames, m_formatComboBox->currentText(), true));
 	}
 	return newdir;
 }
