@@ -29,12 +29,12 @@
 
 #include <qdir.h>
 #include <qstring.h>
+#include <qtextcodec.h>
 #include "qtcompatmac.h"
 #if QT_VERSION >= 0x040000
 #include <QByteArray>
 #endif
 
-#include "standardtags.h"
 #include "genres.h"
 #include "dirinfo.h"
 #include "kid3.h"
@@ -84,6 +84,76 @@
 #include "taglibext/aac/aacfiletyperesolver.h"
 #include "taglibext/mp2/mp2filetyperesolver.h"
 
+
+/**
+ * Data encoding in ID3v1 tags.
+ */
+class TextCodecStringHandler : public TagLib::ID3v1::StringHandler {
+public:
+	/**
+	 * Constructor.
+	 */
+  TextCodecStringHandler() {}
+
+	/**
+	 * Destructor.
+	 */
+	virtual ~TextCodecStringHandler() {}
+
+	/**
+	 * Decode a string from data.
+	 *
+	 * @param data data to decode
+	 */
+	virtual TagLib::String parse(const TagLib::ByteVector& data) const;
+
+	/**
+	 * Encode a byte vector with the data from a string.
+	 *
+	 * @param s string to encode
+	 */
+	virtual TagLib::ByteVector render(const TagLib::String& s) const;
+
+	/**
+	 * Set text codec.
+	 * @param codec text codec, 0 for default behavior (ISO 8859-1)
+	 */
+	static void setTextCodec(const QTextCodec* codec) { s_codec = codec; }
+
+private:
+  static const QTextCodec* s_codec;
+};
+
+const QTextCodec* TextCodecStringHandler::s_codec = 0;
+
+/**
+ * Decode a string from data.
+ *
+ * @param data data to decode
+ */
+TagLib::String TextCodecStringHandler::parse(const TagLib::ByteVector& data) const
+{
+	return s_codec ?
+		QSTRING_TO_TSTRING(s_codec->toUnicode(data.data(), data.size())).stripWhiteSpace() :
+		TagLib::String(data, TagLib::String::Latin1).stripWhiteSpace();
+}
+
+/**
+ * Encode a byte vector with the data from a string.
+ *
+ * @param s string to encode
+ */
+TagLib::ByteVector TextCodecStringHandler::render(const TagLib::String& s) const
+{
+	if (s_codec) {
+		QCM_QCString ba(s_codec->fromUnicode(TStringToQString(s)));
+		return TagLib::ByteVector(ba.data(), ba.size());
+	} else {
+		return s.data(TagLib::String::Latin1);
+	}
+}
+
+
 /** Default text encoding */
 TagLib::String::Type TagLibFile::s_defaultTextEncoding = TagLib::String::Latin1;
 
@@ -420,7 +490,13 @@ QString TagLibFile::getCommentV1()
 {
 	if (m_tagV1) {
 		TagLib::String str = m_tagV1->comment();
-		return str.isNull() ? QString("") : TStringToQString(str);
+		if (str.isNull()) {
+			return QString("");
+		} else {
+			QString qstr(TStringToQString(str));
+			qstr.truncate(28);
+			return qstr;
+		}
 	} else {
 		return QString::null;
 	}
@@ -698,7 +774,7 @@ void TagLibFile::setTitleV1(const QString& str)
 		TagLib::String tstr = str.isEmpty() ?
 			TagLib::String::null : QSTRING_TO_TSTRING(str);
 		if (!(tstr == m_tagV1->title())) {
-			QString s = checkTruncation(str, StandardTags::TF_Title);
+			QString s = checkTruncation(str, 1 << Frame::FT_Title);
 			if (!s.isNull())
 				m_tagV1->setTitle(QSTRING_TO_TSTRING(s));
 			else
@@ -719,7 +795,7 @@ void TagLibFile::setArtistV1(const QString& str)
 		TagLib::String tstr = str.isEmpty() ?
 			TagLib::String::null : QSTRING_TO_TSTRING(str);
 		if (!(tstr == m_tagV1->artist())) {
-			QString s = checkTruncation(str, StandardTags::TF_Artist);
+			QString s = checkTruncation(str, 1 << Frame::FT_Artist);
 			if (!s.isNull())
 				m_tagV1->setArtist(QSTRING_TO_TSTRING(s));
 			else
@@ -740,7 +816,7 @@ void TagLibFile::setAlbumV1(const QString& str)
 		TagLib::String tstr = str.isEmpty() ?
 			TagLib::String::null : QSTRING_TO_TSTRING(str);
 		if (!(tstr == m_tagV1->album())) {
-			QString s = checkTruncation(str, StandardTags::TF_Album);
+			QString s = checkTruncation(str, 1 << Frame::FT_Album);
 			if (!s.isNull())
 				m_tagV1->setAlbum(QSTRING_TO_TSTRING(s));
 			else
@@ -761,7 +837,7 @@ void TagLibFile::setCommentV1(const QString& str)
 		TagLib::String tstr = str.isEmpty() ?
 			TagLib::String::null : QSTRING_TO_TSTRING(str);
 		if (!(tstr == m_tagV1->comment())) {
-			QString s = checkTruncation(str, StandardTags::TF_Comment, 28);
+			QString s = checkTruncation(str, 1 << Frame::FT_Comment, 28);
 			if (!s.isNull())
 				m_tagV1->setComment(QSTRING_TO_TSTRING(s));
 			else
@@ -795,7 +871,7 @@ void TagLibFile::setTrackNumV1(int num)
 {
 	if (makeTagV1Settable() && num >= 0) {
 		if (num != static_cast<int>(m_tagV1->track())) {
-			int n = checkTruncation(num, StandardTags::TF_Track);
+			int n = checkTruncation(num, 1 << Frame::FT_Track);
 			if (n != -1)
 				m_tagV1->setTrack(n);
 			else
@@ -821,7 +897,7 @@ void TagLibFile::setGenreV1(const QString& str)
 		}
 		// if the string cannot be converted to a number, set the truncation flag
 		checkTruncation(!str.isEmpty() && Genres::getNumber(str) == 0xff ? 1 : 0,
-										StandardTags::TF_Genre, 0);
+										1 << Frame::FT_Genre, 0);
 	}
 }
 
@@ -1400,15 +1476,15 @@ static const struct TypeStrOfId {
 	{ Frame::FT_EncodedBy,      I18N_NOOP("TENC - Encoded by"), true },
 	{ Frame::FT_Lyricist,       I18N_NOOP("TEXT - Lyricist/Text writer"), true },
 	{ Frame::FT_Other,          I18N_NOOP("TFLT - File type"), true },
-	{ Frame::FT_Other,          I18N_NOOP("TIPL - Involved people list"), true },
-	{ Frame::FT_Other,          I18N_NOOP("TIT1 - Content group description"), true },
+	{ Frame::FT_Arranger,       I18N_NOOP("TIPL - Involved people list"), true },
+	{ Frame::FT_Grouping,       I18N_NOOP("TIT1 - Content group description"), true },
 	{ Frame::FT_Title,          I18N_NOOP("TIT2 - Title/songname/content description"), true },
 	{ Frame::FT_Subtitle,       I18N_NOOP("TIT3 - Subtitle/Description refinement"), true },
 	{ Frame::FT_Other,          I18N_NOOP("TKEY - Initial key"), true },
 	{ Frame::FT_Language,       I18N_NOOP("TLAN - Language(s)"), true },
 	{ Frame::FT_Other,          I18N_NOOP("TLEN - Length"), true },
-	{ Frame::FT_Other,          I18N_NOOP("TMCL - Musician credits list"), true },
-	{ Frame::FT_Other,          I18N_NOOP("TMED - Media type"), true },
+	{ Frame::FT_Performer,      I18N_NOOP("TMCL - Musician credits list"), true },
+	{ Frame::FT_Media,          I18N_NOOP("TMED - Media type"), true },
 	{ Frame::FT_Other,          I18N_NOOP("TMOO - Mood"), true },
 	{ Frame::FT_OriginalAlbum,  I18N_NOOP("TOAL - Original album/movie/show title"), true },
 	{ Frame::FT_Other,          I18N_NOOP("TOFN - Original filename"), true },
@@ -1416,9 +1492,9 @@ static const struct TypeStrOfId {
 	{ Frame::FT_OriginalArtist, I18N_NOOP("TOPE - Original artist(s)/performer(s)"), true },
 	{ Frame::FT_Other,          I18N_NOOP("TOWN - File owner/licensee"), true },
 	{ Frame::FT_Artist,         I18N_NOOP("TPE1 - Lead performer(s)/Soloist(s)"), true },
-	{ Frame::FT_Performer,      I18N_NOOP("TPE2 - Band/orchestra/accompaniment"), true },
+	{ Frame::FT_AlbumArtist,    I18N_NOOP("TPE2 - Band/orchestra/accompaniment"), true },
 	{ Frame::FT_Conductor,      I18N_NOOP("TPE3 - Conductor/performer refinement"), true },
-	{ Frame::FT_Arranger,       I18N_NOOP("TPE4 - Interpreted, remixed, or otherwise modified by"), true },
+	{ Frame::FT_Remixer,        I18N_NOOP("TPE4 - Interpreted, remixed, or otherwise modified by"), true },
 	{ Frame::FT_Disc,           I18N_NOOP("TPOS - Part of a set"), true },
 	{ Frame::FT_Other,          I18N_NOOP("TPRO - Produced notice"), true },
 	{ Frame::FT_Publisher,      I18N_NOOP("TPUB - Publisher"), true },
@@ -1434,7 +1510,7 @@ static const struct TypeStrOfId {
 	{ Frame::FT_Other,          I18N_NOOP("TXXX - User defined text information"), true },
 	{ Frame::FT_Other,          I18N_NOOP("UFID - Unique file identifier"), true },
 	{ Frame::FT_Other,          I18N_NOOP("USER - Terms of use"), false },
-	{ Frame::FT_Other,          I18N_NOOP("USLT - Unsynchronized lyric/text transcription"), true },
+	{ Frame::FT_Lyrics,         I18N_NOOP("USLT - Unsynchronized lyric/text transcription"), true },
 	{ Frame::FT_Other,          I18N_NOOP("WCOM - Commercial information"), true },
 	{ Frame::FT_Other,          I18N_NOOP("WCOP - Copyright/Legal information"), true },
 	{ Frame::FT_Other,          I18N_NOOP("WOAF - Official audio file webpage"), true },
@@ -1527,7 +1603,9 @@ static QString getFieldsFromTextFrame(
 		TagLib::StringList slText = tFrame->fieldList();
 		text = slText.size() > 1 ? TStringToQString(slText[1]) : "";
 	} else {
-		text = TStringToQString(tFrame->toString());
+		// if there are multiple items, put them into one string
+		// separated by a special separator.
+		text = TStringToQString(tFrame->fieldList().toString(Frame::stringListSeparator()));
 	}
 	field.m_id = Frame::Field::ID_Text;
 	if (type == Frame::FT_Genre) {
@@ -2114,6 +2192,33 @@ void setValue(TagLib::ID3v2::AttachedPictureFrame* f, const TagLib::String& text
 	f->setDescription(text);
 }
 
+static void setStringOrList(TagLib::ID3v2::TextIdentificationFrame* f, const TagLib::String& text)
+{
+	if (text.find(Frame::stringListSeparator()) == -1) {
+		f->setText(text);
+	} else {
+		f->setText(TagLib::StringList::split(text, Frame::stringListSeparator()));
+	}
+}
+
+template <>
+void setValue(TagLib::ID3v2::TextIdentificationFrame* f, const TagLib::String& text)
+{
+	setStringOrList(f, text);
+}
+
+template <class T>
+void setText(T* f, const TagLib::String& text)
+{
+	f->setText(text);
+}
+
+template <>
+void setText(TagLib::ID3v2::TextIdentificationFrame* f, const TagLib::String& text)
+{
+	setStringOrList(f, text);
+}
+
 /**
  * Set the fields in a TagLib ID3v2 frame.
  *
@@ -2149,7 +2254,7 @@ void setTagLibFrame(const TagLibFile* self, T* tFrame, const Frame& frame)
 					} else if (frame.getType() == Frame::FT_Track) {
 						self->addTotalNumberOfTracksIfEnabled(value);
 					}
-					tFrame->setText(QSTRING_TO_TSTRING(value));
+					setText(tFrame, QSTRING_TO_TSTRING(value));
 					break;
 				}
 				case Frame::Field::ID_TextEnc:
@@ -2277,82 +2382,6 @@ void TagLibFile::setId3v2Frame(
 	}
 }
 
-/**
- * Set a frame in the tags 2.
- *
- * @param frame frame to set
- *
- * @return true if ok.
- */
-bool TagLibFile::setFrameV2(const Frame& frame)
-{
-	// If the frame has an index, change that specific frame
-	int index = frame.getIndex();
-	if (index != -1 && m_tagV2) {
-		TagLib::ID3v2::Tag* id3v2Tag;
-		TagLib::Ogg::XiphComment* oggTag;
-		TagLib::APE::Tag* apeTag;
-		if ((id3v2Tag = dynamic_cast<TagLib::ID3v2::Tag*>(m_tagV2)) != 0) {
-			const TagLib::ID3v2::FrameList& frameList = id3v2Tag->frameList();
-			if (index < static_cast<int>(frameList.size())) {
-				// This is a hack. The frameList should not be modified directly.
-				// However when removing the old frame and adding a new frame,
-				// the indices of all frames get invalid.
-				setId3v2Frame(frameList[index], frame);
-				markTag2Changed();
-				return true;
-			}
-		} else if ((oggTag = dynamic_cast<TagLib::Ogg::XiphComment*>(m_tagV2)) != 0) {
-			TagLib::String key = QSTRING_TO_TSTRING(frame.getName(true));
-			TagLib::String value = QSTRING_TO_TSTRING(frame.getValue());
-#if TAGLIB_VERSION <= 0x010400
-			// Remove all fields with that key, because TagLib <= 1.4 crashes
-			// using an invalidated iterator after calling erase().
-			oggTag->addField(key, value, true);
-#else
-			const TagLib::Ogg::FieldListMap& fieldListMap = oggTag->fieldListMap();
-			if (fieldListMap.contains(key) && fieldListMap[key].size() > 1) {
-				int i = 0;
-				TagLib::String oldValue(TagLib::String::null);
-				for (TagLib::Ogg::FieldListMap::ConstIterator it = fieldListMap.begin();
-						 it != fieldListMap.end();
-						 ++it) {
-					TagLib::StringList stringList = (*it).second;
-					for (TagLib::StringList::ConstIterator slit = stringList.begin();
-							 slit != stringList.end();
-							 ++slit) {
-						if (i++ == index) {
-							oldValue = *slit;
-							break;
-						}
-					}
-				}
-				oggTag->removeField(key, oldValue);
-				oggTag->addField(key, value, false);
-			} else {
-				oggTag->addField(key, value, true);
-			}
-#endif
-			if (frame.getType() == Frame::FT_Track) {
-				int numTracks = getTotalNumberOfTracksIfEnabled();
-				if (numTracks > 0) {
-					oggTag->addField("TRACKTOTAL", TagLib::String::number(numTracks), true);
-				}
-			}
-			markTag2Changed();
-			return true;
-		} else if ((apeTag = dynamic_cast<TagLib::APE::Tag*>(m_tagV2)) != 0) {
-			apeTag->addValue(QSTRING_TO_TSTRING(frame.getName(true)),
-											 QSTRING_TO_TSTRING(frame.getValue()));
-			markTag2Changed();
-			return true;
-		}
-	}
-
-	// Try the superclass method
-	return TaggedFile::setFrameV2(frame);
-}
-
 /**
  * Get name of frame from type.
  *
@@ -2371,6 +2400,7 @@ static const char* getVorbisNameFromType(Frame::Type type)
 		"TRACKNUMBER",     // FT_Track,
 		"GENRE",           // FT_Genre,
 		                   // FT_LastV1Frame = FT_Track,
+		"ALBUMARTIST",     // FT_AlbumArtist,
 		"ARRANGER",        // FT_Arranger,
 		"AUTHOR",          // FT_Author,
 		"BPM",             // FT_Bpm,
@@ -2379,9 +2409,12 @@ static const char* getVorbisNameFromType(Frame::Type type)
 		"COPYRIGHT",       // FT_Copyright,
 		"DISCNUMBER",      // FT_Disc,
 		"ENCODED-BY",      // FT_EncodedBy,
+		"GROUPING",        // FT_Grouping,
 		"ISRC",            // FT_Isrc,
 		"LANGUAGE",        // FT_Language,
 		"LYRICIST",        // FT_Lyricist,
+		"LYRICS",          // FT_Lyrics,
+		"SOURCEMEDIA",     // FT_Media,
 		"ORIGINALALBUM",   // FT_OriginalAlbum,
 		"ORIGINALARTIST",  // FT_OriginalArtist,
 		"ORIGINALDATE",    // FT_OriginalDate,
@@ -2389,6 +2422,7 @@ static const char* getVorbisNameFromType(Frame::Type type)
 		"PERFORMER",       // FT_Performer,
 		"UNKNOWN",         // FT_Picture,
 		"PUBLISHER",       // FT_Publisher,
+		"REMIXER",         // FT_Remixer,
 		"SUBTITLE",        // FT_Subtitle,
 		"WEBSITE",         // FT_Website,
 		                   // FT_LastFrame = FT_Website
@@ -2487,6 +2521,88 @@ static QString getApeName(const Frame& frame)
 	}
 }
 
+/**
+ * Set a frame in the tags 2.
+ *
+ * @param frame frame to set
+ *
+ * @return true if ok.
+ */
+bool TagLibFile::setFrameV2(const Frame& frame)
+{
+	// If the frame has an index, change that specific frame
+	int index = frame.getIndex();
+	if (index != -1 && m_tagV2) {
+		TagLib::ID3v2::Tag* id3v2Tag;
+		TagLib::Ogg::XiphComment* oggTag;
+		TagLib::APE::Tag* apeTag;
+		if ((id3v2Tag = dynamic_cast<TagLib::ID3v2::Tag*>(m_tagV2)) != 0) {
+			const TagLib::ID3v2::FrameList& frameList = id3v2Tag->frameList();
+			if (index < static_cast<int>(frameList.size())) {
+				// This is a hack. The frameList should not be modified directly.
+				// However when removing the old frame and adding a new frame,
+				// the indices of all frames get invalid.
+				setId3v2Frame(frameList[index], frame);
+				markTag2Changed();
+				return true;
+			}
+		} else if ((oggTag = dynamic_cast<TagLib::Ogg::XiphComment*>(m_tagV2)) != 0) {
+			if (frame.getType() == Frame::FT_Picture) {
+				return false;
+			}
+			TagLib::String key = QSTRING_TO_TSTRING(getVorbisName(frame));
+			TagLib::String value = QSTRING_TO_TSTRING(frame.getValue());
+#if TAGLIB_VERSION <= 0x010400
+			// Remove all fields with that key, because TagLib <= 1.4 crashes
+			// using an invalidated iterator after calling erase().
+			oggTag->addField(key, value, true);
+#else
+			const TagLib::Ogg::FieldListMap& fieldListMap = oggTag->fieldListMap();
+			if (fieldListMap.contains(key) && fieldListMap[key].size() > 1) {
+				int i = 0;
+				TagLib::String oldValue(TagLib::String::null);
+				for (TagLib::Ogg::FieldListMap::ConstIterator it = fieldListMap.begin();
+						 it != fieldListMap.end();
+						 ++it) {
+					TagLib::StringList stringList = (*it).second;
+					for (TagLib::StringList::ConstIterator slit = stringList.begin();
+							 slit != stringList.end();
+							 ++slit) {
+						if (i++ == index) {
+							oldValue = *slit;
+							break;
+						}
+					}
+				}
+				oggTag->removeField(key, oldValue);
+				oggTag->addField(key, value, false);
+			} else {
+				oggTag->addField(key, value, true);
+			}
+#endif
+			if (frame.getType() == Frame::FT_Track) {
+				int numTracks = getTotalNumberOfTracksIfEnabled();
+				if (numTracks > 0) {
+					oggTag->addField("TRACKTOTAL", TagLib::String::number(numTracks), true);
+				}
+			}
+			markTag2Changed();
+			return true;
+		} else if ((apeTag = dynamic_cast<TagLib::APE::Tag*>(m_tagV2)) != 0) {
+			if (frame.getType() == Frame::FT_Picture) {
+				return false;
+			}
+			apeTag->addValue(QSTRING_TO_TSTRING(getApeName(frame)),
+											 QSTRING_TO_TSTRING(frame.getValue()));
+			markTag2Changed();
+			return true;
+		}
+	}
+
+	// Try the superclass method
+	return TaggedFile::setFrameV2(frame);
+}
+
 /**
  * Check if an ID3v2.4.0 frame ID is valid.
  *
@@ -2562,6 +2678,9 @@ bool TagLibFile::addFrameV2(Frame& frame)
 				}
 			} else if (frameId == "USLT") {
 				id3Frame = new TagLib::ID3v2::UnsynchronizedLyricsFrame(enc);
+				if (id3Frame) {
+					((TagLib::ID3v2::UnsynchronizedLyricsFrame*)id3Frame)->setLanguage("eng");
+				}
 			}
 			if (!id3Frame) {
 				TagLib::ID3v2::UserTextIdentificationFrame* txxxFrame =
@@ -2600,6 +2719,9 @@ bool TagLibFile::addFrameV2(Frame& frame)
 				return true;
 			}
 		} else if ((oggTag = dynamic_cast<TagLib::Ogg::XiphComment*>(m_tagV2)) != 0) {
+			if (frame.getType() == Frame::FT_Picture) {
+				return false;
+			}
 			QString name(getVorbisName(frame));
 			TagLib::String tname = QSTRING_TO_TSTRING(name);
 			TagLib::String tvalue = QSTRING_TO_TSTRING(frame.getValue());
@@ -2630,6 +2752,9 @@ bool TagLibFile::addFrameV2(Frame& frame)
 			markTag2Changed();
 			return true;
 		} else if ((apeTag = dynamic_cast<TagLib::APE::Tag*>(m_tagV2)) != 0) {
+			if (frame.getType() == Frame::FT_Picture) {
+				return false;
+			}
 			QString name(getApeName(frame));
 			TagLib::String tname = QSTRING_TO_TSTRING(name);
 			TagLib::String tvalue = QSTRING_TO_TSTRING(frame.getValue());
@@ -2888,12 +3013,11 @@ QStringList TagLibFile::getFrameIds() const
 		for (unsigned i = 0; i < sizeof(typeStrOfId) / sizeof(typeStrOfId[0]); ++i) {
 			const TypeStrOfId& ts = typeStrOfId[i];
 			if (ts.type == Frame::FT_Other && ts.supported && ts.str) {
-				lst.append(ts.str);
+				lst.append(QCM_translate(ts.str));
 			}
 		}
 	} else {
 		static const char* const fieldNames[] = {
-			"ALBUMARTIST",
 			"CATALOGNUMBER",
 			"CONTACT",
 			"DESCRIPTION",
@@ -2913,11 +3037,9 @@ QStringList TagLibFile::getFrameIds() const
 			"PRODUCTNUMBER",
 			"RECORDINGDATE",
 			"RELEASE DATE",
-			"REMIXER",
 			"SOURCE ARTIST",
 			"SOURCE MEDIUM",
 			"SOURCE WORK",
-			"SOURCEMEDIA",
 			"SPARS",
 			"TRACKTOTAL",
 			"VERSION",
@@ -2943,6 +3065,18 @@ void TagLibFile::staticInit()
 #endif
 	TagLib::FileRef::addFileTypeResolver(new AACFileTypeResolver);
 	TagLib::FileRef::addFileTypeResolver(new MP2FileTypeResolver);
+
+	TagLib::ID3v1::Tag::setStringHandler(new TextCodecStringHandler);
+}
+
+/**
+ * Set the text codec to be used for tag 1.
+ *
+ * @param codec text codec, 0 to use default (ISO 8859-1)
+ */
+void TagLibFile::setTextCodecV1(const QTextCodec* codec)
+{
+	TextCodecStringHandler::setTextCodec(codec);
 }
 
 /**
@@ -2985,7 +3119,7 @@ TaggedFile* TagLibFile::Resolver::createFile(const DirInfo* di,
 			 && Kid3App::s_miscCfg.m_id3v2Version == MiscConfig::ID3v2_4_0
 #endif
 				)
-			|| ext == ".mpc" || ext == ".ogg" || ext == "flac"
+			|| ext == ".mpc" || ext == ".oga" || ext == ".ogg" || ext == "flac"
 			|| ext == ".spx" || ext == ".tta"
 			|| ext.right(3) == ".wv")
 		return new TagLibFile(di, fn);
@@ -3000,7 +3134,7 @@ TaggedFile* TagLibFile::Resolver::createFile(const DirInfo* di,
  */
 QStringList TagLibFile::Resolver::getSupportedFileExtensions() const
 {
-	return QStringList() << ".flac" << ".mp3" << ".mpc" << ".ogg" <<
+	return QStringList() << ".flac" << ".mp3" << ".mpc" << ".oga" << ".ogg" <<
 		".spx" << ".tta" << ".aac" << ".mp2" << ".wv";
 }
 
